@@ -29,6 +29,15 @@
   Leftrightarrowlim Longleftarrowlim Longrightarrowlim Longleftrightarrowlim
   cdotslim
 
+  ;; further arrows
+  threeleftarrows threerightarrows
+  fourleftarrows fourrightarrows
+  longleftrightarrows longleftleftarrows
+  longthreeleftarrows longthreerightarrows
+  longrightleftarrows longrightrightarrows
+  longfourleftarrows longfourrightarrows
+  LRleftrightarrow Llongleftarrow Llongrightarrow Llongleftrightarrow
+
   ;; rotated arrows and other symbols
   mapsfrom longmapsfrom mapmulti leftsquigarrow
   upequal downequal longupequal longdownequal longupminus longdownminus
@@ -49,7 +58,8 @@
   divides ndivides asterisk dottimes precdot
 
   ;; extra literal symbols
-  mathcatalan mathd mathD mathe matheuler mathlambda mathi mathpi
+  mathcatalan mathd mathD mathe matheuler
+  mathGamma mathlambda mathLaplace mathi mathpi
   Alpha Beta Epsilon Eta Iota Kappa Mu Nu Omicron Chi Rho Tau Zeta
 
   ;; negations
@@ -71,8 +81,10 @@
   assign plusassign minusassign timesassign overassign backassign
   lflux gflux colons transtype
   lebar gebar leangle geangle leqangle geqangle
+  anglele anglege legeangle geleangle
   udots subsetsim supsetsim
-  rightmap leftmap leftrightmap)
+  rightmap leftmap leftrightmap
+  tmxspace)
 
 (logic-rules
   ((latex-texmacs-arity% 'x 0) (latex-texmacs-symbol% 'x))
@@ -84,23 +96,28 @@
 
 (logic-group latex-texmacs-0%
   tmunsc emdash tmhrule tmat tmbsl tmdummy
-  TeXmacs madebyTeXmacs withTeXmacstext tmmade
-  scheme tmsep tmSep pari qed textdots hrule)
+  TeXmacs madebyTeXmacs withTeXmacstext citewebsite tmmade
+  scheme tmsep tmSep pari qed textdots hrule filldots
+  infixand infixor infixiff)
 
 (logic-group latex-texmacs-1%
-  key tmrsub tmrsup keepcase
+  citetexmacs key tmrsub tmrsup keepcase
   tmtextrm tmtextsf tmtexttt tmtextmd tmtextbf
   tmtextup tmtextsl tmtextit tmtextsc tmmathbf tmmathmd
   tmverbatim tmop tmstrong tmem tmtt tmname tmsamp tmabbr
   tmdfn tmkbd tmvar tmacronym tmperson tmscript tmdef
-  dueto op tmdate tmoutput tmerrput tmtiming
+  dueto op todo tmdate tmoutput tmerrput tmtiming
   tmsubtitle tmrunningtitle tmrunningauthor
   tmaffiliation tmemail tmhomepage
   tmfnaffiliation tmfnemail tmfnhomepage
-  tmacmhomepage tmacmmisc tmieeeemail tmnote tmmisc)
-
-(logic-group latex-texmacs-1*%
-  tmcodeinline)
+  tmacmhomepage tmacmmisc tmieeeemail tmnote tmmisc
+  uhat uwidehat utilde uwidetilde
+  uvec ubreve uinvbreve ucheck uring uacute ugrave
+  underdot uddot udddot uddddot
+  widespacing nonconverted
+  tmcodeinline groupcommoncounter
+  ;; NOTE: for personal use from vdh style package
+  gb gbt)
 
 (logic-group latex-texmacs-2%
   tmcolor
@@ -116,11 +133,13 @@
   tmfoldedplain tmunfoldedplain tmfoldedenv tmunfoldedenv
   tmfoldedstd tmunfoldedstd tmfolded tmunfolded
   tminput tminputmath tmhlink tmaction ontop subindex
-  renderfootnote tmlinenumber)
+  renderfootnote tmlinenumber
+  addtocountergroup)
 
 (logic-group latex-texmacs-3%
   tmsession tmfoldedio tmunfoldedio tmfoldediomath tmunfoldediomath
-  tmlinenote subsubindex tmref glossaryentry natbib-triple)
+  tmlinenote subsubindex tmref glossaryentry natbib-triple
+  renderfootnotestar)
 
 (logic-group latex-texmacs-4%
   tmscriptinput tmscriptoutput tmconverterinput tmconverteroutput
@@ -159,6 +178,10 @@
   ("tmcode" 0)
   ("tmparmod" 3)
   ("tmparsep" 1)
+  ("tmcompact" 0)
+  ("tmcompressed" 0)
+  ("tmamplified" 0)
+  ("tmjumpin" 0)
   ("tmindent" 0)
   ("tmlisting" 0)
   ("elsequation" 0)
@@ -170,6 +193,7 @@
 
 (logic-group latex-texmacs-environment-0%
   begin-proof begin-leftaligned begin-rightaligned begin-quoteenv
+  begin-tmcompact begin-tmcompressed begin-tmamplified begin-tmjumpin
   begin-tmindent begin-tmlisting begin-elsequation begin-elsequation*)
 
 (logic-group latex-texmacs-environment-0*%
@@ -244,7 +268,7 @@
   "tmpadded" "tmoverlined" "tmunderlined" "tmbothlined"
   "tmframed" "tmornamented")
 
-(logic-group latex-texmacs-env-preamble-environment%
+(logic-group latex-texmacs-theorem-environment%
   "theorem" "proposition" "lemma" "corollary"
   "axiom" "definition" "notation" "conjecture"
   "remark" "note" "example" "convention"
@@ -271,6 +295,8 @@
   begin-exercise* begin-problem* begin-solution*)
 
 (logic-rules
+  ((latex-texmacs-env-preamble-environment% 'x)
+   (latex-texmacs-theorem-environment% 'x))
   ((latex-texmacs-arity% 'x 0) (latex-texmacs-theorem% 'x))
   ((latex-environment-0% 'x) (latex-texmacs-theorem% 'x)))
 
