@@ -26,6 +26,7 @@
 (define-public widget-style-button 32)
 (define-public widget-style-centered 64)
 (define-public widget-style-bold 128)
+(define-public widget-style-verb 256)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Control structures
@@ -58,6 +59,12 @@
   (:synopsis "When primitive for content generation")
   `(cons* 'list (if ,pred? ($list ,@l) '())))
 
+(tm-define-macro ($for* var-vals . l)
+  (:synopsis "For primitive for content generation")
+  `(list 'for
+         (lambda (,(car var-vals)) ($list ,@l))
+         (lambda () ,(cadr var-vals))))
+
 (tm-define (cond$sub l)
   (cond ((null? l)
          (list `(else '())))
@@ -84,8 +91,10 @@
 
 (tm-define-macro ($with var val . l)
   (:synopsis "With primitive for content generation")
-  `(with ,var ,val
-     (cons* 'list ($list ,@l))))
+  (if (string? var)
+      ($quote `(with ,var ,val ($unquote ($inline ,@l))))
+      `(with ,var ,val
+         (cons* 'list ($list ,@l)))))
 
 (tm-define-macro ($execute cmd . l)
   (:synopsis "Execute one command")
@@ -255,11 +264,17 @@
   `(list ,text (lambda () ,@cmds)))
 
 (tm-define-macro ($check text check pred?)
-  (:synopsis "Make button")
+  (:synopsis "Make check")
   (if developer-mode?
     (ahash-set! all-translations text #t))
   `(list 'check ,text ,check (lambda () ,pred?)))
 
+(tm-define-macro ($shortcut* text sh)
+  (:synopsis "Make shortcut")
+  (if developer-mode?
+    (ahash-set! all-translations text #t))
+  `(list 'shortcut ,text ,sh))
+
 (tm-define-macro ($balloon text balloon)
   (:synopsis "Make balloon")
   (if developer-mode?
@@ -300,6 +315,10 @@
     (ahash-set! all-translations text #t))
   `(list 'text ,(process-translate text)))
 
+(tm-define-macro ($menu-invisible text)
+  (:synopsis "Make invisible")
+  `(list 'invisible ,text))
+
 (tm-define-macro ($input cmd type proposals width)
   (:synopsis "Make input field")
   `(list 'input (lambda (answer) ,cmd) ,type (lambda () ,proposals) ,width))
@@ -350,9 +369,14 @@
 
 (tm-define form-name "empty")
 (tm-define form-entries (list))
+(tm-define form-text-entries (list))
 (tm-define form-last (make-ahash-table))
 
 (tm-define (form-named-set name field val)
+  (if (and (pair? val) (pair? (cdr val)) (cadr val)) (set! val (car val)))
+  (ahash-set! form-last (list name field) val))
+
+(tm-define (form-named-set-multiple name field val)
   (ahash-set! form-last (list name field) val))
 
 (tm-define (form-named-ref name field)
@@ -373,7 +397,8 @@
 (tm-define-macro ($form name . l)
   (:synopsis "Make form")
   `($let* ((form-name ,name)
-           (form-entries (list)))
+           (form-entries (list))
+           (form-text-entries (list)))
      ,@l))
 
 (tm-define (form-proposals name field l)
@@ -388,9 +413,13 @@
 (tm-define-macro ($form-input field type proposals width)
   (:synopsis "Make a textual input field for the current form")
   `($execute
-     (set! form-entries (append form-entries (list ,field)))
+     (begin
+       (set! form-entries (append form-entries (list ,field)))
+       (set! form-text-entries (append form-text-entries (list ,field))))
      ($input (form-named-set form-name ,field answer)
-             ,type (form-proposals form-name ,field ,proposals) ,width)))
+             (with nr (number->string (length form-text-entries))
+               (string-append ,field "#form-" form-name "-" nr ":" ,type))
+	     (form-proposals form-name ,field ,proposals) ,width)))
 
 (tm-define-macro ($form-enum field proposals selected width)
   (:synopsis "Make an enumeration field for the current form")
@@ -402,7 +431,7 @@
 
 (tm-define-macro ($form-choice field proposals selected) 
   (:synopsis "Make a single choice field for the current form") 
-  `($execute 
+  `($execute
      (set! form-entries (append form-entries (list ,field))) 
      ($choice (form-named-set form-name ,field answer)
               (form-proposals-sel form-name ,field ,proposals ,selected)
@@ -410,12 +439,20 @@
 
 (tm-define-macro ($form-choices field proposals selected) 
   (:synopsis "Make a multiple choice field for the current form") 
-  `($execute 
+  `($execute
      (set! form-entries (append form-entries (list ,field))) 
-     ($choices (form-named-set form-name ,field answer)
-               (form-proposals-sel form-name ,field ,proposals ,selected)
+     ($choices (form-named-set-multiple form-name ,field answer)
+               (begin
+                 (form-named-set-multiple form-name ,field ,selected)
+                 ,proposals)
                ,selected)))
 
+(tm-define-macro ($form-toggle field on?)
+  (:synopsis "Make a toggle field for the current form") 
+  `($execute
+     (set! form-entries (append form-entries (list ,field))) 
+     ($toggle (form-named-set form-name ,field answer) ,on?)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Basic text markup
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -516,6 +553,9 @@
 (tm-define-macro ($link dest . l)
   ($quote `(hlink ($unquote ($inline ,@l)) ($unquote ($textual ,dest)))))
 
+(tm-define-macro ($color col . l)
+  ($quote `(with "color" ,col ($unquote ($inline ,@l)))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Specific markup for TeXmacs documentation
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -589,6 +629,144 @@
 (tm-define-macro ($src-arg s)
   ($quote `(src-arg ($unquote ($textual ,s)))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Graphics
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define-macro ($geometry w h unit . l)
+  `(list 'with
+         "gr-geometry" (list 'tuple "geometry" ,w ,h "center")
+         "gr-frame" (list 'tuple "scale" ,unit
+                          (list 'tuple "0.5gw" "0.5gh"))
+         ($inline ,@l)))
+
+(tm-define-macro ($auto-crop . l)
+  `(list 'with
+         "gr-auto-crop" "true"
+         ($inline ,@l)))
+
+(tm-define-macro ($grid unit . l)
+  `(list 'with
+         "gr-grid" (list 'tuple "cartesian" (list 'point "0" "0")
+                         ,(markup-build-coordinate unit))
+         ($inline ,@l)))
+
+(define (build-with w x)
+  (if (tm-func? x 'with)
+      `(with ,@w ,@(cdr x))
+      `(with ,@w ,x)))
+
+(tm-define (markup-build-graphics-items l)
+  (cond ((null? l) (list))
+        ((tm-func? (car l) 'concat)
+         (append (markup-build-graphics-items (cdar l))
+                 (markup-build-graphics-items (cdr l))))
+        ((tm-func? (car l) 'with)
+         (let* ((head (cDr (cdar l)))
+                (tail (cAr (car l)))
+                (sl (if (tm-func? tail 'concat) (cdr tail) (list tail)))
+                (sr (map markup-build-graphics-items sl))
+                (sr* (map (lambda (x) (build-with head x)) sr))
+                (r (markup-build-graphics-items (cdr l))))
+           (append sr* r)))
+        (else (cons (car l) (markup-build-graphics-items (cdr l))))))
+
+(tm-define (markup-build-graphics l)
+  (with x (append-map markup-expand-document l)
+    (cons 'graphics (markup-build-graphics-items x))))
+
+(tm-define-macro ($graphics . l)
+  `(markup-build-graphics ($list ,@l)))
+
+(tm-define (markup-build-coordinate x)
+  (cond ((string? x) x)
+        ((number? x)
+         (if (exact? x)
+             (number->string (exact->inexact x))
+             (number->string x)))
+        (else "0")))
+
+(tm-define (markup-build-point l)
+  (with x (append-map markup-expand-document l)
+    (cons 'point (map markup-build-coordinate x))))
+
+(tm-define-macro ($point . l)
+  `(markup-build-point ($list ,@l)))
+ 
+(tm-define-macro ($line . l)
+  `(cons 'line ($list ,@l)))
+
+(tm-define-macro ($cline . l)
+  `(cons 'cline ($list ,@l)))
+
+(tm-define-macro ($spline . l)
+  `(cons 'spline ($list ,@l)))
+
+(tm-define-macro ($cspline . l)
+  `(cons 'cspline ($list ,@l)))
+
+(tm-define-macro ($arc . l)
+  `(cons 'arc ($list ,@l)))
+
+(tm-define-macro ($carc . l)
+  `(cons 'carc ($list ,@l)))
+
+(tm-define-macro ($text-at p . l)
+  ($quote `(text-at ($unquote ($inline ,@l)) ($unquote ($inline ,p)))))
+
+(tm-define-macro ($math-at p . l)
+  ($quote `(math-at ($unquote ($inline ,@l)) ($unquote ($inline ,p)))))
+
+(tm-define (markup-build-graphical l)
+  (with x (append-map markup-expand-document l)
+    (if (== (length x) 1) (car x)
+        (cons 'gr-group x))))
+
+(tm-define-macro ($graphical . l)
+  `(markup-build-graphical ($list ,@l)))
+
+(tm-define-macro ($line-width w . l)
+  ($quote `(with "line-width" ,w ($unquote ($graphical ,@l)))))
+
+(tm-define-macro ($pen-color col . l)
+  ($quote `(with "color" ,col ($unquote ($graphical ,@l)))))
+
+(tm-define-macro ($fill-color col . l)
+  ($quote `(with "fill-color" ,col ($unquote ($graphical ,@l)))))
+
+(tm-define-macro ($text-align h v . l)
+  ($quote `(with "text-at-halign" ,h
+                 "text-at-valign" ,v
+                 ($unquote ($inline ,@l)))))
+
+(tm-define-macro ($graph2d x1 x2 steps fun)
+  `($line
+     ($for (_k_ (.. 0 (+ ,steps 1)))
+       ($let* ((f ,fun)
+               (dx (/ (- ,x2 ,x1) ,steps))
+               (x (+ ,x1 (* _k_ dx)))
+               (y (f x)))
+         ($point x y)))))
+
+(tm-define-macro ($curve2d t1 t2 steps xt yt)
+  `($line
+     ($for (_k_ (.. 0 (+ ,steps 1)))
+       ($let* ((fx ,xt)
+               (fy ,yt)
+               (dt (/ (- ,t2 ,t1) ,steps))
+               (t (+ ,t1 (* _k_ dt)))
+               (x (fx t))
+               (y (fy t)))
+         ($point x y)))))
+
+(tm-define (markup-build-animation duration l)
+  (with x (append-map markup-expand-document l)
+    (cons 'anim-compose
+          (map (lambda (f) `(anim-constant ,f ,duration)) x))))
+
+(tm-define-macro ($animation duration . l)
+  `(markup-build-animation ,duration ($list ,@l)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; User interface for dynamic content generation
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
