@@ -46,6 +46,14 @@
   (require-format x '(:%1 :%2 :*))
   `(,(car x) ,(cadr x) ,(caddr x) (menu-dynamic ,@(cdddr x))))
 
+(define (gui-make-push-focus x)
+  (require-format x '(push-focus :%1 :*))
+  `(with pushed-tree ,(cadr x)
+     (with pushed-focus (tree->fingerprint pushed-tree)
+       (menu-dynamic
+         (invisible (tree->path pushed-tree))
+         ,@(cddr x)))))
+
 (define (gui-make-cond x)
   (require-format x '(cond :*))
   (with fun (lambda (x)
@@ -79,6 +87,10 @@
   (require-format x '(text :%1))
   `($menu-text ,(cadr x)))
 
+(define (gui-make-invisible x)
+  (require-format x '(invisible :%1))
+  `($menu-invisible ,(cadr x)))
+
 (define (gui-make-glue x)
   (require-format x '(glue :%4))
   `($glue ,(second x) ,(third x) ,(fourth x) ,(fifth x)))
@@ -143,6 +155,10 @@
   (require-format x '(check :%3))
   `($check ,(gui-make (cadr x)) ,(caddr x) ,(cadddr x)))
 
+(define (gui-make-shortcut x)
+  (require-format x '(shortcut :%2))
+  `($shortcut* ,(gui-make (cadr x)) ,(caddr x)))
+
 (define (gui-make-balloon x)
   (require-format x '(balloon :%2))
   `($balloon ,(gui-make (cadr x)) ,(gui-make (caddr x))))
@@ -219,6 +235,10 @@
   (require-format x '(mono :*))
   `($widget-style ,widget-style-monospaced ,@(map gui-make (cdr x))))
 
+(define (gui-make-verb x)
+  (require-format x '(verb :*))
+  `($widget-style ,widget-style-verb ,@(map gui-make (cdr x))))
+
 (define (gui-make-tile x)
   (require-format x '(tile :integer? :*))
   `($tile ,(cadr x) ,@(map gui-make (cddr x))))
@@ -347,6 +367,7 @@
   (let ,gui-make-let)
   (let* ,gui-make-let)
   (with ,gui-make-with)
+  (push-focus ,gui-make-push-focus)
   (receive ,gui-make-with)
   (cond ,gui-make-cond)
   (loop ,gui-make-loop)
@@ -354,6 +375,7 @@
   (refreshable ,gui-make-refreshable)
   (group ,gui-make-group)
   (text ,gui-make-text)
+  (invisible ,gui-make-invisible)
   (glue ,gui-make-glue)
   (color ,gui-make-color)
   (texmacs-output ,gui-make-texmacs-output)
@@ -370,6 +392,7 @@
   (concat ,gui-make-concat)
   (verbatim ,gui-make-verbatim)
   (check ,gui-make-check)
+  (shortcut ,gui-make-shortcut)
   (balloon ,gui-make-balloon)
   (-> ,gui-make-submenu)
   (=> ,gui-make-top-submenu)
@@ -389,6 +412,7 @@
   (bold ,gui-make-bold)
   (grey ,gui-make-grey)
   (mono ,gui-make-monospaced)
+  (verb ,gui-make-verb)
   (tile ,gui-make-tile)
   (scrollable ,gui-make-scrollable)
   (resize ,gui-make-resize)
